package uk.ac.vamsas.client.picking;

import java.io.*;
import java.net.*;
import java.util.logging.*;

import org.apache.commons.logging.Log;

class PickEndPoint extends Thread
{
	private Log logger = org.apache.commons.logging.LogFactory.getLog(uk.ac.vamsas.client.picking.PickEndPoint.class);
	
	private Socket socket;
	private int rPort;
	private BufferedWriter os;
	private BufferedReader in;

	private SocketManager manager;	
	
	PickEndPoint(SocketManager manager, Socket s)
	{
		this.manager = manager;
		socket = s;
	}
	
	boolean openConnection()
	{
		try
		{
			// Create the socket if it doesn't already exist
			if (socket == null)
				socket = new Socket(InetAddress.getLocalHost(), PickServer.PORT);
			
			rPort = socket.getPort();
			socket.setKeepAlive(true);
						
			// Open the streams for reading/writing
			os = new BufferedWriter(new OutputStreamWriter(socket.getOutputStream()));
			in = new BufferedReader(new InputStreamReader(socket.getInputStream()));
		
			// Start the thread to listen for incoming messages
//			logger.info("CLIENT: connection successful to port "
//				+ socket.getPort() + " via port " + socket.getLocalPort());
			start();
			
			return true;
		}
		catch (Exception e)
		{
			//logger.info("CLIENT: connection failed: " + e);
			return false;
		}
	}
	
	void send(Message message)
	{
		try
		{
			String str = message.getRawMessage();
			
			//logger.info("CLIENT: send " + str + " to " + rPort);
			os.write(str);
			
			// We use a newline to terminate the message
			os.newLine();
			os.flush();
		}
		catch (Exception e)
		{
			//logger.info("CLIENT: failed to send");
			
			// TODO: terminate the connection on a failed send or retry?
			terminate();
		}
	}
  /**
   * state of server thread - running or not running
   */
  private boolean running=false;
  /**
   * condition for server thread to stop
   */
  private boolean enabled=true;
  public void haltEndpoint() {
  // TODO: FIX THIS METHOD
    if (!running || !enabled) {
      logger.debug("PickEndpoint is not running or already requested to halt.");
      return;
    }
    logger.debug("Disabling pick endpoint.");
    enabled=false;
    terminate();
    while (running) {
      try {
        Thread.sleep(1);
      } catch (Exception e) {
      }
    }
    logger.debug("Pick endpoint has stopped.");
  }

	// void receive() (threaded)
	public void run()
	{
    enabled=true;
    running=true;
		try
		{
			while (enabled)
			{
				String str = in.readLine();				
				//logger.info("CLIENT: recv " + str + " from " + rPort);
				
				// TODO: Spawn this off into the GUI Event-Dispatch thread...
				
				// Convert the string back into something API friendly
        if (str!=null) {
				Message message = strToMessage(str);
				
				// Ignore corrupted or unknown message types
				if (message != null)				
					manager.processMessage(this, message);
        }
			}
		}
		catch (Exception e)
		{
			// Means the other end of the connection has (probably) died
      logger.debug("Exception in receiver loop: "+e.getMessage());
    }
		// terminate this endpoint (if this is server side)
    //logger.info("CLIENT: read failed: " + e);
          
    terminate();
  
		running=false;
	}
	
	private void terminate()
	{
		try { 
      if (socket.isConnected())
        socket.close(); 
      }
		catch (IOException e) {}
		
		//logger.info("CLIENT: closing connection to port " + socket.getPort());
		manager.removeEndPoint(this);
	}
	
	private Message strToMessage(String str)
	{
    // TODO: generalize message class 
		try
		{
			if (str.startsWith("CUSTOM"))
				return new CustomMessage(str.substring(6));
		
			if (str.startsWith("MOUSEOVER"))
				return new MouseOverMessage(str);
		}
		catch (Exception e)
		{
			//logger.info("Unable to reconstruct message: " + e);
		}
		
		return null;
	}
}